/* 
    kleds for KDE2

    Copyright (C) 2001 Hans Matzen (hans@hansmatzen.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//
// main program for kleds
//

#define VERSION "0.5 (for KDE2)"



extern "C" {
#include <pthread.h>
#include <stdlib.h>
#include <unistd.h>
}

#include "kleds.h"
#include "xkbwatch.h"

#include <qobject.h>
#include <kuniqueapp.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <kglobal.h>
#include <klocale.h>
#include <dcopclient.h>
#include <qcstring.h>
#include <qdatastream.h>

static const char *description = 
       I18N_NOOP("A keyboard LED indicator for kicker.");
static const char *version = VERSION;


// this is main1 the thread function for the gui
void *main1(void *display) 
{
    // init kapp stuff
    // fake it for KCmdLineArgs
    int argc=1;
    char *argv[argc];
    argv[0]="kleds";
    
    KAboutData aboutData("kleds", "KLeds", version, description,
			 KAboutData::License_GPL,
			 "(c) 2001, Hans Matzen", 
			 I18N_NOOP("KLeds is a keyboard LED indicator for KDE2.\nIt is usefull if you use a keyboard without LED's\nsuch as the Logitech Cordless Desktop keyboard\nor just for fun because you like it."), 
			 "http://www.hansmatzen.de", "hans@hansmatzen.de");
    
    aboutData.addAuthor("Hans Matzen", I18N_NOOP("KLeds Developer"), 
			"hans@hansmatzen.de");
    KCmdLineArgs::init(argc,argv,&aboutData);
      
    KApplication app;
    KLeds *k = new KLeds(0,"KLeds");
    QCString appid;

    //app.setTopWidget( (QWidget *) k);
    app.setMainWidget( (QWidget *) k);

    // register with dcop

    if ( !app.dcopClient()->isRegistered() ) {
	appid=app.dcopClient()->registerAs("kleds",false);
	//app.dcopClient()->setDefaultObject( k->objId() );
    }

    // to prevent a button in the taskbar
    QByteArray params;
    QDataStream d(params, IO_WriteOnly);
    d << (int) getpid();
    
    app.dcopClient()->send("kicker", "TaskbarApplet", "clientDied(pid_t)",
    		 params);
    
    // request current ledstate for init from xkb watcher
    app.dcopClient()->send("kleds","kledssender","get_indicators()","");
    
    // start it
    app.exec();
    return 0;
}

// this is main2 the thread function for the xkb watcher
void *main2(void *displaystr)
{
    xkbwatch *xkbw = new xkbwatch((char *) displaystr);
    xkbw->run();
    return 0;
}


int main() 
{
    // get DISPLAY env var
    const void * disp=getenv("DISPLAY");

    // if DISPLAY is not set try the default
    if (!disp)
	disp = ":0.0";

    pthread_t thread1;
    pthread_t thread2;

    // start xkb watcher thread
    pthread_create(&thread1,NULL, main2, (void *) disp);
    // start gui thread
    pthread_create(&thread2,NULL, main1, (void *) disp); 

    // do not join the xkb thread we want to terminate 
    // if the gui is closed pthread_join(thread1,NULL);
    pthread_join(thread2,NULL);

    exit(0);
}


